@@ -6,7 +6,7 @@ Uses syntax highlighting for presenting the various information elements.
 Similar in spirit to the inspect module, but all calls take a name argument to
 reference the name under which an object is being read.
 
-$Id: OInspect.py 1329 2006-05-26 07:52:45Z fperez $
+$Id: OInspect.py 1850 2006-10-28 19:48:13Z fptest $
 """
 
 #*****************************************************************************
@@ -30,6 +30,7 @@ import string
 import StringIO
 import types
 import os
+import sys
 # IPython's own
 from IPython import PyColorize
 from IPython.genutils import page,indent,Term,mkdict
@@ -37,6 +38,45 @@ from IPython.Itpl import itpl
 from IPython.wildcard import list_namespace
 from IPython.ColorANSI import *
 
+#****************************************************************************
+# HACK!!! This is a crude fix for bugs in python 2.3's inspect module.  We
+# simply monkeypatch inspect with code copied from python 2.4.
+if sys.version_info[:2] == (2,3):
+    from inspect import ismodule, getabsfile, modulesbyfile
+    def getmodule(object):
+        """Return the module an object was defined in, or None if not found."""
+        if ismodule(object):
+            return object
+        if hasattr(object, '__module__'):
+            return sys.modules.get(object.__module__)
+        try:
+            file = getabsfile(object)
+        except TypeError:
+            return None
+        if file in modulesbyfile:
+            return sys.modules.get(modulesbyfile[file])
+        for module in sys.modules.values():
+            if hasattr(module, '__file__'):
+                modulesbyfile[
+                    os.path.realpath(
+                            getabsfile(module))] = module.__name__
+        if file in modulesbyfile:
+            return sys.modules.get(modulesbyfile[file])
+        main = sys.modules['__main__']
+        if not hasattr(object, '__name__'):
+            return None
+        if hasattr(main, object.__name__):
+            mainobject = getattr(main, object.__name__)
+            if mainobject is object:
+                return main
+        builtin = sys.modules['__builtin__']
+        if hasattr(builtin, object.__name__):
+            builtinobject = getattr(builtin, object.__name__)
+            if builtinobject is object:
+                return builtin
+
+    inspect.getmodule = getmodule
+
 #****************************************************************************
 # Builtin color schemes
 
@@ -96,6 +136,27 @@ def getdoc(obj):
             ds = '%s\n%s' % (ds,ds2)
     return ds
 
+def getsource(obj,is_binary=False):
+    """Wrapper around inspect.getsource.
+
+    This can be modified by other projects to provide customized source
+    extraction.
+
+    Inputs:
+
+    - obj: an object whose source code we will attempt to extract.
+
+    Optional inputs:
+
+    - is_binary: whether the object is known to come from a binary source.
+    This implementation will skip returning any output for binary objects, but
+    custom extractors may know how to meaninfully process them."""
+    
+    if is_binary:
+        return None
+    else:
+        return inspect.getsource(obj)
+
 #****************************************************************************
 # Class definitions
 
@@ -222,7 +283,7 @@ class Inspector:
         # Flush the source cache because inspect can return out-of-date source
         linecache.checkcache()
         try:
-            src = inspect.getsource(obj) 
+            src = getsource(obj) 
         except:
             self.noinfo('source',oname)
         else:
@@ -362,15 +423,16 @@ class Inspector:
             linecache.checkcache()
             source_success = False
             try:
-                if not binary_file:
-                    source = self.format(inspect.getsource(obj))
+                source = self.format(getsource(obj,binary_file))
+                if source:
                     out.write(header('Source:\n')+source.rstrip())
                     source_success = True
-            except:
+            except Exception, msg:
                 pass
             
             if ds and not source_success:
-                out.writeln(header('Docstring [source file open failed]:\n') + indent(ds))
+                out.writeln(header('Docstring [source file open failed]:\n')
+                            + indent(ds))
 
         # Constructor docstring for classes
         if obj_type is types.ClassType:
