@@ -72,6 +72,9 @@ import re
 import shlex
 import sys
 import IPython.rlineimpl as readline    
+import itertools
+from IPython.ipstruct import Struct
+from IPython import ipapi
 
 import types
 
@@ -198,7 +201,11 @@ class Completer:
         try:
             object = eval(expr, self.namespace)
         except:
-            object = eval(expr, self.global_namespace)
+            try:
+                object = eval(expr, self.global_namespace)
+            except:
+                return []
+                
 
         # Start building the attribute list via dir(), and then complete it
         # with a few extra special-purpose calls.
@@ -328,7 +335,7 @@ class IPCompleter(Completer):
                 for f in self.glob("%s*" % text)]            
 
     def file_matches(self, text):
-        """Match filneames, expanding ~USER type strings.
+        """Match filenames, expanding ~USER type strings.
 
         Most of the seemingly convoluted logic in this completer is an
         attempt to handle filenames with spaces in them.  And yet it's not
@@ -350,6 +357,12 @@ class IPCompleter(Completer):
 
         protectables = ' ()[]{}'
 
+        if text.startswith('!'):
+            text = text[1:]
+            text_prefix = '!'
+        else:
+            text_prefix = ''
+            
         def protect_filename(s):
             return "".join([(ch in protectables and '\\' + ch or ch)
                             for ch in s])
@@ -367,7 +380,7 @@ class IPCompleter(Completer):
                 open_quotes = 1
                 lsplit = lbuf.split("'")[-1]
             else:
-                return None
+                return []
         except IndexError:
             # tab pressed on empty line
             lsplit = ""
@@ -382,7 +395,7 @@ class IPCompleter(Completer):
             text = os.path.expanduser(text)
 
         if text == "":
-            return [protect_filename(f) for f in self.glob("*")]
+            return [text_prefix + protect_filename(f) for f in self.glob("*")]
 
         m0 = self.clean_glob(text.replace('\\',''))
         if has_protectables:
@@ -390,7 +403,8 @@ class IPCompleter(Completer):
             # beginning of filename so that we don't double-write the part
             # of the filename we have so far
             len_lsplit = len(lsplit)
-            matches = [text0 + protect_filename(f[len_lsplit:]) for f in m0]
+            matches = [text_prefix + text0 + 
+                       protect_filename(f[len_lsplit:]) for f in m0]
         else:
             if open_quotes:
                 # if we have a string with an open quote, we don't need to
@@ -398,12 +412,18 @@ class IPCompleter(Completer):
                 # would cause bugs when the filesystem call is made).
                 matches = m0
             else:
-                matches = [protect_filename(f) for f in m0]
+                matches = [text_prefix + 
+                           protect_filename(f) for f in m0]
         if len(matches) == 1 and os.path.isdir(matches[0]):
             # Takes care of links to directories also.  Use '/'
             # explicitly, even under Windows, so that name completions
-            # don't end up escaped.
-            matches[0] += '/'
+            # don't end up escaped.            
+            d = matches[0]
+            if d[-1] in ['/','\\']:
+                d = d[:-1]
+            
+            matches = [ (d + '/' + p) for p in os.listdir(d) ]
+            
         return matches
 
     def alias_matches(self, text):
@@ -526,6 +546,42 @@ class IPCompleter(Completer):
                     argMatches.append("%s=" %namedArg)
         return argMatches
 
+    def dispatch_custom_completer(self,text):
+        # print "Custom! '%s' %s" % (text, self.custom_completers) # dbg
+        line = self.full_lbuf        
+        if not line.strip():
+            return None
+
+        event = Struct()
+        event.line = line
+        event.symbol = text
+        cmd = line.split(None,1)[0]
+        event.command = cmd
+        #print "\ncustom:{%s]\n" % event # dbg
+        
+        # for foo etc, try also to find completer for %foo
+        if not cmd.startswith(self.magic_escape):
+            try_magic = self.custom_completers.s_matches(
+              self.magic_escape + cmd)            
+        else:
+            try_magic = []
+        
+        
+        for c in itertools.chain(
+                                 self.custom_completers.s_matches(cmd),
+                                 try_magic,
+                                 self.custom_completers.flat_matches(self.lbuf)):
+            # print "try",c # dbg
+            try:
+                res = c(event)
+                return [r for r in res if r.lower().startswith(text.lower())]
+            except ipapi.TryNext:
+                pass
+            
+        return None
+        
+    
+    
     def complete(self, text, state):
         """Return the next possible completion for 'text'.
 
@@ -542,11 +598,13 @@ class IPCompleter(Completer):
 
         # don't apply this on 'dumb' terminals, such as emacs buffers, so we
         # don't interfere with their own tab-completion mechanism.
-        self.lbuf = self.get_line_buffer()[:self.readline.get_endidx()]
+        self.full_lbuf = self.get_line_buffer()
+        self.lbuf = self.full_lbuf[:self.readline.get_endidx()]
         if not (self.dumb_terminal or self.get_line_buffer().strip()):
             self.readline.insert_text('\t')
             return None
 
+        
         magic_escape = self.magic_escape
         magic_prefix = self.magic_prefix
 
@@ -556,26 +614,31 @@ class IPCompleter(Completer):
             elif text.startswith('~'):
                 text = os.path.expanduser(text)
             if state == 0:
-                # Extend the list of completions with the results of each
-                # matcher, so we return results to the user from all
-                # namespaces.
-                if self.merge_completions:
-                    self.matches = []
-                    for matcher in self.matchers:
-                        self.matches.extend(matcher(text))
+                custom_res = self.dispatch_custom_completer(text)
+                if custom_res is not None:
+                    # did custom completers produce something?
+                    self.matches = custom_res
                 else:
-                    for matcher in self.matchers:
-                        self.matches = matcher(text)
-                        if self.matches:
-                            break
+                    # Extend the list of completions with the results of each
+                    # matcher, so we return results to the user from all
+                    # namespaces.
+                    if self.merge_completions:
+                        self.matches = []
+                        for matcher in self.matchers:
+                            self.matches.extend(matcher(text))
+                    else:
+                        for matcher in self.matchers:
+                            self.matches = matcher(text)
+                            if self.matches:
+                                break
 
             try:
                 return self.matches[state].replace(magic_prefix,magic_escape)
             except IndexError:
                 return None
         except:
-            #from IPython.ultraTB import AutoFormattedTB; # dbg
-            #tb=AutoFormattedTB('Verbose');tb() #dbg
+            from IPython.ultraTB import AutoFormattedTB; # dbg
+            tb=AutoFormattedTB('Verbose');tb() #dbg
 
             # If completion fails, don't annoy the user.
             return None
