@@ -6,7 +6,7 @@ Requires Python 2.1 or better.
 
 This file contains the main make_IPython() starter function.
 
-$Id: ipmaker.py 1347 2006-06-04 00:42:44Z fperez $"""
+$Id: ipmaker.py 1979 2006-12-12 18:50:20Z vivainio $"""
 
 #*****************************************************************************
 #       Copyright (C) 2001-2006 Fernando Perez. <fperez@colorado.edu>
@@ -170,7 +170,7 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
     # The "ignore" option is a kludge so that Emacs buffers don't crash, since
     # the 'C-c !' command in emacs automatically appends a -i option at the end.
     cmdline_only = ('help ignore|i ipythondir=s Version upgrade '
-                    'gthread! qthread! wthread! pylab! tk!')
+                    'gthread! qthread! q4thread! wthread! pylab! tk!')
 
     # Build the actual name list to be used by DPyGetOpt
     opts_names = qw(cmdline_opts) + qw(cmdline_only)
@@ -208,6 +208,7 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
                       prompt_in2 = '   .\\D.: ',
                       prompt_out = 'Out[\\#]: ',
                       prompts_pad_left = 1,
+                      quiet = 0,
                       quick = 0,
                       readline = 1,
                       readline_merge_completions = 1,
@@ -217,9 +218,11 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
                       separate_in = '\n',
                       separate_out = '\n',
                       separate_out2 = '',
+                      system_header = 'IPython system call: ',
                       system_verbose = 0,
                       gthread = 0,
                       qthread = 0,
+                      q4thread = 0,
                       wthread = 0,
                       pylab = 0,
                       tk = 0,
@@ -573,13 +576,13 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
             import_fail_info(mod)
 
     for mod_fn in IP_rc.import_some:
-        if mod_fn == []: break
-        mod,fn = mod_fn[0],','.join(mod_fn[1:])
-        try:
-            exec 'from '+mod+' import '+fn in IP.user_ns
-        except :
-            IP.InteractiveTB()
-            import_fail_info(mod,fn)
+        if not mod_fn == []: 
+            mod,fn = mod_fn[0],','.join(mod_fn[1:])
+            try:
+                exec 'from '+mod+' import '+fn in IP.user_ns
+            except :
+                IP.InteractiveTB()
+                import_fail_info(mod,fn)
 
     for mod in IP_rc.import_all:
         try:
