@@ -6,13 +6,13 @@
 ;; URL:           http://ipython.scipy.org
 ;; Compatibility: Emacs21, XEmacs21
 ;; FIXME: #$@! INPUT RING
-(defconst ipython-version "$Revision: 1325 $"
+(defconst ipython-version "$Revision: 1851 $"
   "VC version number.")
 
 ;;; Commentary 
 ;; This library makes all the functionality python-mode has when running with
 ;; the normal python-interpreter available for ipython, too. It also enables a
-;; persistent py-shell command history accross sessions (if you exit python
+;; persistent py-shell command history across sessions (if you exit python
 ;; with C-d in py-shell) and defines the command `ipython-to-doctest', which
 ;; can be used to convert bits of a ipython session into something that can be
 ;; used for doctests. To install, put this file somewhere in your emacs
@@ -197,8 +197,10 @@ the second for a 'normal' command, and the third for a multiline command.")
     (setq py-traceback-line-re
           "\\(^[^\t ].+?\\.py\\).*\n   +[0-9]+[^\00]*?\n-+> \\([0-9]+\\) +")
     
-    ;; Recognize the ipython pdb, whose prompt is 'ipdb>' instead of '(Pdb)'
-    (setq py-pdbtrack-input-prompt "\n[(<]*[Ii]?[Pp]db[>)]+ ")
+    ;; Recognize the ipython pdb, whose prompt is 'ipdb>' or  'ipydb>'
+    ;;instead of '(Pdb)'
+    (setq py-pdbtrack-input-prompt "\n[(<]*[Ii]?[Pp]y?db[>)]+ ")
+    (setq py-pydbtrack-input-prompt "\n[(]*ipydb[>)]+ ")
     
     (setq py-shell-input-prompt-1-regexp "^In \\[[0-9]+\\]: *"
           py-shell-input-prompt-2-regexp "^   [.][.][.]+: *" )
@@ -248,9 +250,9 @@ buffer already exists."
 (defadvice py-execute-region (around py-execute-buffer-ensure-process)
   "HACK: if `py-shell' is not active or ASYNC is explicitly desired, fall back
   to python instead of ipython." 
-  (let ((py-python-command (if (and (comint-check-proc "*Python*") (not async))
-                               py-python-command
-                               ipython-backup-of-py-python-command)))
+  (let ((py-which-shell (if (and (comint-check-proc "*Python*") (not async))
+			    py-python-command
+			  ipython-backup-of-py-python-command)))
     ad-do-it))
 (ad-activate 'py-execute-region)
 
@@ -456,23 +458,15 @@ matches last process output."
                           (point-min-marker)))
         (end-marker (process-mark (get-buffer-process (current-buffer))))
         (text (ansi-color-filter-apply (buffer-substring start-marker end-marker))))
-   (progn
-     ;; XXX if `text' matches both pattern, it MUST be the last prompt-2
-     (cond ((and (string-match "\n$" text) (string-match
-py-shell-input-prompt-2-regexp text))
-            (with-current-buffer (ipython-get-indenting-buffer)
-              (erase-buffer)))
-           ;; still a prompt-2
-           ((string-match py-shell-input-prompt-2-regexp text)
-            (with-current-buffer (ipython-get-indenting-buffer)
-              (setq ipython-indentation-string
-                    (buffer-substring (point-at-bol) (point))))
-            (unless (eq ipython-indentation-string nil)
-              (message "ipython-indentation-hook: %s#%s##" text
-ipython-indentation-string))
-            (goto-char end-marker)
-            (insert ipython-indentation-string)))
-           (setq ipython-indentation-string nil))))
+   ;; XXX if `text' matches both pattern, it MUST be the last prompt-2
+   (when (and (string-match py-shell-input-prompt-2-regexp text)
+	      (not (string-match "\n$" text)))
+     (with-current-buffer (ipython-get-indenting-buffer)
+       (setq ipython-indentation-string
+	     (buffer-substring (point-at-bol) (point))))
+     (goto-char end-marker)
+     (insert ipython-indentation-string)
+     (setq ipython-indentation-string nil))))
 
 (add-hook 'py-shell-hook
          (lambda ()
